@@ -1,17 +1,21 @@
-#include "read_matlab4.h"
-#include "write_matlab4.h"
+#include "util/read_matlab4.h"
+#include "util/write_matlab4.h"
 #include <stdint.h>
 #include <string.h>
 #include "errorext.h"
 #include "systemimpl.h"
 #include "ptolemyio.h"
-#include "read_csv.h"
+#include "util/read_csv.h"
 #include <math.h>
 #include <gc.h>
-#include "omc_msvc.h" /* For INFINITY and NAN */
-#if !defined(__MINGW32__) && !defined(_MSC_VER)
+#include "util/omc_file.h"
+#include "util/omc_msvc.h" /* For INFINITY and NAN */
 #include <time.h>
+#include <sys/types.h>
 #include <sys/stat.h>
+
+#if defined(__MINGW32__) || defined(_MSC_VER)
+#define stat _stat
 #endif
 
 #if defined(_MSC_VER)
@@ -31,9 +35,7 @@ const char *PlotFormatStr[] = {"Unknown","MATLAB4","PLT","CSV"};
 typedef struct {
   PlotFormat curFormat;
   char *curFileName;
-#if !defined(__MINGW32__) && !defined(_MSC_VER)
   time_t mtime;
-#endif
   ModelicaMatReader matReader;
   FILE *pltReader;
   struct csv_data *csvReader;
@@ -62,18 +64,17 @@ static PlotFormat SimulationResultsImpl__openFile(const char *filename, Simulati
   PlotFormat format;
   int len = strlen(filename);
   const char *msg[] = {"",""};
-#if !defined(__MINGW32__) && !defined(_MSC_VER)
+#if defined(__MINGW32__) || defined(_MSC_VER)
+  struct _stat buf;
+#else
   struct stat buf = {0} /* Zero this or valgrind complains */;
 #endif
+
   if (simresglob->curFileName && 0==strcmp(filename,simresglob->curFileName)) {
-#if defined(__MINGW32__) || defined(_MSC_VER)
-    return simresglob->curFormat; // Super cache :)
-#else
     /* Also check that the file was not modified */
-    if (stat(filename, &buf)==0 && difftime(buf.st_mtime,simresglob->mtime)==0.0) {
+    if (omc_stat(filename, &buf)==0 && difftime(buf.st_mtime,simresglob->mtime)==0.0) {
       return simresglob->curFormat; // Super cache :)
     }
-#endif
   }
   // Start by closing the old file...
   SimulationResultsImpl__close(simresglob);
@@ -96,7 +97,7 @@ static PlotFormat SimulationResultsImpl__openFile(const char *filename, Simulati
     }
     break;
   case PLT:
-    simresglob->pltReader = fopen(filename, "r");
+    simresglob->pltReader = omc_fopen(filename, "r");
     if (simresglob->pltReader==NULL) {
       msg[1] = filename;
       c_add_message(NULL,-1, ErrorType_scripting, ErrorLevel_error, gettext("Failed to open simulation result %s: %s"), msg, 2);
@@ -120,7 +121,7 @@ static PlotFormat SimulationResultsImpl__openFile(const char *filename, Simulati
   simresglob->curFormat = format;
   simresglob->curFileName = strdup(filename);
 #if !defined(__MINGW32__) && !defined(_MSC_VER)
-  stat(filename, &buf);
+  omc_stat(filename, &buf);
   simresglob->mtime = buf.st_mtime;
 #endif
   // fprintf(stderr, "SimulationResultsImpl__openFile(%s) => %s\n", filename, PlotFormatStr[curFormat]);
@@ -392,12 +393,18 @@ static void* SimulationResultsImpl__readDataset(const char *filename, void *vars
   }
 }
 
-static inline int failedToWriteToFile(const char *file)
+static inline int failedToWriteToFileImpl(const char *file, const char *sourceFile, const char *line)
 {
-  c_add_message(NULL,-1, ErrorType_scripting, ErrorLevel_error, gettext("Failed to write to file %s."), &file, 1);
+  const char *msg[3] = {file,line,sourceFile};
+  c_add_message(NULL,-1, ErrorType_scripting, ErrorLevel_error, gettext("%s:%s: Failed to write to file %s."), msg, 3);
   return 0;
 }
 
+#define STR_HELPER(x) #x
+#define STR(x) STR_HELPER(x)
+#define failedToWriteToFile(file) failedToWriteToFileImpl(file, __FILE__, STR(__LINE__))
+
+
 static inline int intMax(int a, int b)
 {
   return a>b ? a : b;
@@ -412,7 +419,7 @@ static int endsWith(const char *s, const char *suffix)
     return 0;
   }
 }
-int SimulationResults_filterSimulationResults(const char *inFile, const char *outFile, void *vars, int numberOfIntervals)
+int SimulationResults_filterSimulationResults(const char *inFile, const char *outFile, void *vars, int numberOfIntervals, int removeDescription, int readAllVars)
 {
   const char *msg[5] = {"","","","",""};
   void *tmp;
@@ -435,11 +442,13 @@ int SimulationResults_filterSimulationResults(const char *inFile, const char *ou
     int *parameter_indexesToOutput = NULL;
     FILE *fout = NULL;
     char *tmp;
-    double start_stop[2] = {0};
     double start = omc_matlab4_startTime(&simresglob.matReader);
     double stop = omc_matlab4_stopTime(&simresglob.matReader);
+    double start_stop[2] = {start, stop};
     parameter_indexes[0] = 1; /* time */
-    omc_matlab4_read_all_vals(&simresglob.matReader);
+    if (readAllVars) {
+      omc_matlab4_read_all_vals(&simresglob.matReader);
+    }
     if (endsWith(outFile,".csv")) {
       double **vals = omc_alloc_interface.malloc(sizeof(double*)*numToFilter);
       FILE *fout = NULL;
@@ -461,7 +470,7 @@ int SimulationResults_filterSimulationResults(const char *inFile, const char *ou
           vals[i] = omc_matlab4_read_vals(&simresglob.matReader, mat_var[i]->index);
         }
       }
-      fout = fopen(outFile, "w");
+      fout = omc_fopen(outFile, "w");
       fprintf(fout, "time");
       for (i=1; i<numToFilter; i++) {
         fprintf(fout, ",\"%s\"", mat_var[i]->name);
@@ -521,7 +530,7 @@ int SimulationResults_filterSimulationResults(const char *inFile, const char *ou
       /* indexes becomes the lookup table from old index to new index */
       parameter_indexes[i] = j;
     }
-    fout = fopen(outFile, "wb");
+    fout = omc_fopen(outFile, "wb");
     if (fout == NULL) {
       return failedToWriteToFile(outFile);
     }
@@ -545,21 +554,27 @@ int SimulationResults_filterSimulationResults(const char *inFile, const char *ou
     }
     GC_free(tmp);
 
-    if (writeMatVer4MatrixHeader(fout, "description", numToFilter, longestDesc, sizeof(int8_t))) {
-      return failedToWriteToFile(outFile);
-    }
-
-    tmp = omc_alloc_interface.malloc(numToFilter*longestDesc);
-    for (i=0; i<numToFilter; i++) {
-      int len = strlen(mat_var[i]->descr);
-      for (j=0; j<len; j++) {
-        tmp[numToFilter*j+i] = mat_var[i]->descr[j];
+    if (removeDescription) {
+      if (writeMatVer4MatrixHeader(fout, "description", numToFilter, 0, sizeof(int8_t))) {
+        return failedToWriteToFile(outFile);
       }
+    } else {
+      if (writeMatVer4MatrixHeader(fout, "description", numToFilter, longestDesc, sizeof(int8_t))) {
+        return failedToWriteToFile(outFile);
+      }
+
+      tmp = omc_alloc_interface.malloc(numToFilter*longestDesc);
+      for (i=0; i<numToFilter; i++) {
+        int len = strlen(mat_var[i]->descr);
+        for (j=0; j<len; j++) {
+          tmp[numToFilter*j+i] = mat_var[i]->descr[j];
+        }
+      }
+      if (1 != fwrite(tmp, numToFilter*longestDesc, 1, fout)) {
+        return failedToWriteToFile(outFile);
+      }
+      GC_free(tmp);
     }
-    if (1 != fwrite(tmp, numToFilter*longestDesc, 1, fout)) {
-      return failedToWriteToFile(outFile);
-    }
-    GC_free(tmp);
 
     if (writeMatVer4MatrixHeader(fout, "dataInfo", numToFilter, 4, sizeof(int32_t))) {
       return failedToWriteToFile(outFile);
@@ -592,10 +607,6 @@ int SimulationResults_filterSimulationResults(const char *inFile, const char *ou
     if (writeMatVer4MatrixHeader(fout, "data_1", 2, numUniqueParam, sizeof(double))) {
       return failedToWriteToFile(outFile);
     }
-    start = omc_matlab4_startTime(&simresglob.matReader);
-    stop = omc_matlab4_stopTime(&simresglob.matReader);
-    start_stop[0]=start;
-    start_stop[1]=stop;
 
     if (1 != fwrite(start_stop, sizeof(double)*2, 1, fout)) {
       return failedToWriteToFile(outFile);
@@ -628,10 +639,10 @@ int SimulationResults_filterSimulationResults(const char *inFile, const char *ou
         }
       }
       msg[4] = inFile;
-      GC_asprintf((char**)msg+3, "%d", simresglob.matReader.nrows);
-      GC_asprintf((char**)msg+2, "%d", numberOfIntervals);
-      GC_asprintf((char**)msg+1, "%d", nevents);
-      GC_asprintf((char**)msg+0, "%d", neventpoints);
+      GC_asprintf(msg+3, "%d", simresglob.matReader.nrows);
+      GC_asprintf(msg+2, "%d", numberOfIntervals);
+      GC_asprintf(msg+1, "%d", nevents);
+      GC_asprintf(msg+0, "%d", neventpoints);
       c_add_message(NULL,-1, ErrorType_scripting, ErrorLevel_notification, gettext("Resampling %s from %s points to %s points, removing %s events stored in %s points.\n"), msg, 5);
     }
 
@@ -654,8 +665,8 @@ int SimulationResults_filterSimulationResults(const char *inFile, const char *ou
           var.index=indexesToOutput[i];
           if (omc_matlab4_val(vals+j, &simresglob.matReader, &var, t)) {
             msg[2] = inFile;
-            GC_asprintf((char**)msg+1, "%d", indexesToOutput[i]);
-            GC_asprintf((char**)msg+0, "%.15g", t);
+            GC_asprintf(msg+1, "%d", indexesToOutput[i]);
+            GC_asprintf(msg+0, "%.15g", t);
             c_add_message(NULL,-1, ErrorType_scripting, ErrorLevel_error, gettext("Resampling %s failed to get variable %s at time %s.\n"), msg, 3);
             return 0;
           }
@@ -664,8 +675,11 @@ int SimulationResults_filterSimulationResults(const char *inFile, const char *ou
         vals = omc_matlab4_read_vals(&simresglob.matReader, indexesToOutput[i]);
         nrows = simresglob.matReader.nrows;
       }
-      if (1!=fwrite(vals, sizeof(double)*nrows, 1, fout)) {
-        return failedToWriteToFile(outFile);
+      if (nrows > 0) {
+        if (1!=fwrite(vals, sizeof(double)*nrows, 1, fout)) {
+          fprintf(stderr, "nrows=%d\n", nrows);
+          return failedToWriteToFile(outFile);
+        }
       }
       if (numberOfIntervals) {
         GC_free(vals);
