@@ -156,7 +156,7 @@ int getAnalyticalJacobianLis(DATA* data, threadData_t *threadData, int sysNumber
   ANALYTIC_JACOBIAN* parentJacobian = systemData->parDynamicData[omc_get_thread_num()].parentJacobian;
 
   int nth = 0;
-  int nnz = jacobian->sparsePattern->numberOfNoneZeros;
+  int nnz = jacobian->sparsePattern->numberOfNonZeros;
 
   for(i=0; i < jacobian->sizeRows; i++) {
     jacobian->seedVars[i] = 1;
