@@ -73,6 +73,8 @@ int initializeLinearSystems(DATA *data, threadData_t *threadData)
   int res;
   unsigned int j, maxNumberThreads;
   LINEAR_SYSTEM_DATA *linsys = data->simulationInfo->linearSystemData;
+  modelica_boolean someSmallDensity = 0;  /* pretty dumping of flag info */
+  modelica_boolean someBigSize = 0;       /* analogous to someSmallDensity */
 
   maxNumberThreads = omc_get_max_threads();
 
@@ -80,7 +82,7 @@ int initializeLinearSystems(DATA *data, threadData_t *threadData)
   infoStreamPrint(LOG_LS, 0, "%ld linear systems", data->modelData->nLinearSystems);
 
   if (LSS_DEFAULT == data->simulationInfo->lssMethod) {
-#ifdef WITH_UMFPACK
+#ifdef WITH_SUITESPARSE
     data->simulationInfo->lssMethod = LSS_KLU;
 #elif !defined(OMC_MINIMAL_RUNTIME)
     data->simulationInfo->lssMethod = LSS_LIS;
@@ -116,7 +118,7 @@ int initializeLinearSystems(DATA *data, threadData_t *threadData)
         linsys[i].jacobianIndex = -1;
         throwStreamPrint(threadData, "Failed to initialize the jacobian for torn linear system %d.", (int)linsys[i].equationIndex);
       }
-      nnz = jacobian->sparsePattern->numberOfNoneZeros;
+      nnz = jacobian->sparsePattern->numberOfNonZeros;
       linsys[i].nnz = nnz;
 
 #ifdef USE_PARJAC
@@ -125,7 +127,6 @@ int initializeLinearSystems(DATA *data, threadData_t *threadData)
       {
         // ToDo Simplify this. Only have one location for jacobian
         linsys[i].parDynamicData[j].jacobian = (ANALYTIC_JACOBIAN*) malloc(sizeof(ANALYTIC_JACOBIAN));
-        //linsys[i].initialAnalyticalJacobian(data, threadData, linsys[i].parDynamicData[j].jacobian);
         linsys[i].parDynamicData[j].jacobian->sizeCols = jacobian->sizeCols;
         linsys[i].parDynamicData[j].jacobian->sizeRows = jacobian->sizeRows;
         linsys[i].parDynamicData[j].jacobian->sizeTmpVars = jacobian->sizeTmpVars;
@@ -140,10 +141,30 @@ int initializeLinearSystems(DATA *data, threadData_t *threadData)
 #endif
     }
 
-    if(nnz/(double)(size*size)<=linearSparseSolverMaxDensity && size>=linearSparseSolverMinSize)
-    {
+    if (nnz/(double)(size*size) < linearSparseSolverMaxDensity) {
       linsys[i].useSparseSolver = 1;
-      infoStreamPrint(LOG_STDOUT, 0, "Using sparse solver for linear system %d,\nbecause density of %.3f remains under threshold of %.3f and size of %d exceeds threshold of %d.\nThe maximum density and the minimal system size for using sparse solvers can be specified\nusing the runtime flags '<-lssMaxDensity=value>' and '<-lssMinSize=value>'.", i, nnz/(double)(size*size), linearSparseSolverMaxDensity, size, linearSparseSolverMinSize);
+      someSmallDensity = 1;
+      if (size > linearSparseSolverMinSize) {
+        someBigSize = 1;
+        infoStreamPrint(LOG_STDOUT, 0,
+                        "Using sparse solver for linear system %d,\n"
+                        "because density of %.3f remains under threshold of %.3f\n"
+                        "and size of %d exceeds threshold of %d.",
+                        i, nnz/(double)(size*size), linearSparseSolverMaxDensity,
+                        size, linearSparseSolverMinSize);
+      } else {
+        infoStreamPrint(LOG_STDOUT, 0,
+                        "Using sparse solver for linear system %d,\n"
+                        "because density of %.3f remains under threshold of %.3f.",
+                        i, nnz/(double)(size*size), linearSparseSolverMaxDensity);
+      }
+    } else if (size > linearSparseSolverMinSize) {
+      linsys[i].useSparseSolver = 1;
+      someBigSize = 1;
+        infoStreamPrint(LOG_STDOUT, 0,
+                        "Using sparse solver for linear system %d,\n"
+                        "because size of %d exceeds threshold of %d.",
+                        i, size, linearSparseSolverMinSize);
     }
 
     /* allocate more system data */
@@ -159,7 +180,7 @@ int initializeLinearSystems(DATA *data, threadData_t *threadData)
     {
       switch(data->simulationInfo->lssMethod)
       {
-    #ifdef WITH_UMFPACK
+    #ifdef WITH_SUITESPARSE
       case LSS_UMFPACK:
         linsys[i].setAElement = setAElementUmfpack;
         linsys[i].setBElement = setBElement;
@@ -196,7 +217,7 @@ int initializeLinearSystems(DATA *data, threadData_t *threadData)
         throwStreamPrint(threadData, "OMC is compiled without sparse linear solver Lis.");
         break;
     #endif
-    #if defined(OMC_MINIMAL_RUNTIME) && !defined(WITH_UMFPACK)
+    #if defined(OMC_MINIMAL_RUNTIME) && !defined(WITH_SUITESPARSE)
       case LSS_DEFAULT:
         {
           int indexes[2] = {1, linsys[i].equationIndex};
@@ -209,9 +230,10 @@ int initializeLinearSystems(DATA *data, threadData_t *threadData)
         throwStreamPrint(threadData, "unrecognized sparse linear solver (%d)", data->simulationInfo->lssMethod);
       }
     }
-    if(linsys[i].useSparseSolver == 0) { /* Not an else-statement because there might not be a sparse linear solver available */
-    switch(data->simulationInfo->lsMethod)
+    if(linsys[i].useSparseSolver == 0) /* Not an else-statement because there might not be a sparse linear solver available */
     {
+      switch(data->simulationInfo->lsMethod)
+      {
       case LS_LAPACK:
         linsys[i].setAElement = setAElement;
         linsys[i].setBElement = setBElement;
@@ -233,7 +255,7 @@ int initializeLinearSystems(DATA *data, threadData_t *threadData)
         }
         break;
     #endif
-    #ifdef WITH_UMFPACK
+    #ifdef WITH_SUITESPARSE
       case LS_UMFPACK:
         linsys[i].setAElement = setAElementUmfpack;
         linsys[i].setBElement = setBElement;
@@ -283,6 +305,20 @@ int initializeLinearSystems(DATA *data, threadData_t *threadData)
     }
   }
 
+  /* print relevant flag information */
+  if(someSmallDensity) {
+    if(someBigSize) {
+      infoStreamPrint(LOG_STDOUT, 0, "The maximum density and the minimal system size for using sparse solvers can be\n"
+                                     "specified using the runtime flags '<-lssMaxDensity=value>' and '<-lssMinSize=value>'.");
+    } else {
+      infoStreamPrint(LOG_STDOUT, 0, "The maximum density for using sparse solvers can be specified\n"
+                                     "using the runtime flag '<-lssMaxDensity=value>'.");
+    }
+  } else if(someBigSize) {
+    infoStreamPrint(LOG_STDOUT, 0, "The minimal system size for using sparse solvers can be specified\n"
+                                   "using the runtime flag '<-lssMinSize=value>'.");
+  }
+
   messageClose(LOG_LS);
 
   TRACE_POP
@@ -335,6 +371,11 @@ int updateStaticDataOfLinearSystems(DATA *data, threadData_t *threadData)
 
   for(i=0; i<data->modelData->nLinearSystems; ++i)
   {
+    // check if LS is initialized
+    if (linsys[i].nominal == NULL || linsys[i].min == NULL || linsys[i].max==NULL)
+    {
+      throwStreamPrint(threadData, "Static data of Linear system not initialized for linear system %i",i);
+    }
     linsys[i].initializeStaticLSData(data, threadData, &linsys[i]);
   }
 
@@ -380,13 +421,9 @@ int freeLinearSystems(DATA *data, threadData_t *threadData)
   for(i=0; i<data->modelData->nLinearSystems; ++i)
   {
     /* free system and solver data */
-    for (j=0; j<omc_get_max_threads(); ++j)
-    {
-      free(linsys[i].parDynamicData[j].b);
-    }
-    free(linsys[i].nominal);
-    free(linsys[i].min);
-    free(linsys[i].max);
+    free(linsys[i].nominal); linsys[i].nominal = NULL;
+    free(linsys[i].min); linsys[i].min = NULL;
+    free(linsys[i].max); linsys[i].max = NULL;
 
     /* ToDo Implement unique function to free a ANALYTIC_JACOBIAN */
     if (1 == linsys[i].method) {
@@ -394,6 +431,18 @@ int freeLinearSystems(DATA *data, threadData_t *threadData)
       freeAnalyticJacobian(jacobian);
       /* Note: The Jacobian of data->simulationInfo itself will be free later. */
 
+    if (linsys[i].parDynamicData == NULL)
+    {
+      break;
+    }
+    else
+    {
+      for (j=0; j<omc_get_max_threads(); ++j)
+      {
+        free(linsys[i].parDynamicData[j].b); linsys[i].parDynamicData[j].b = NULL;
+      }
+    }
+
 #ifdef USE_PARJAC
       for (j=0; j<omc_get_max_threads(); ++j) {
         // Note: We cannot use neither freeAnalyticJacobian() nor freeSparsePattern()
@@ -422,7 +471,7 @@ int freeLinearSystems(DATA *data, threadData_t *threadData)
         break;
     #endif
 
-    #ifdef WITH_UMFPACK
+    #ifdef WITH_SUITESPARSE
       case LSS_UMFPACK:
         for (j=0; j<omc_get_max_threads(); ++j)
         {
@@ -462,7 +511,7 @@ int freeLinearSystems(DATA *data, threadData_t *threadData)
         break;
   #endif
 
-  #ifdef WITH_UMFPACK
+  #ifdef WITH_SUITESPARSE
       case LS_UMFPACK:
         for (j=0; j<omc_get_max_threads(); ++j)
         {
@@ -498,7 +547,7 @@ int freeLinearSystems(DATA *data, threadData_t *threadData)
         break;
 
       default:
-        throwStreamPrint(threadData, "unrecognized dense linear solver (data->simulationInfo->lsMethod)");
+        throwStreamPrint(threadData, "unrecognized dense linear solver (%d)", data->simulationInfo->lsMethod);
       }
     }
 
@@ -545,7 +594,7 @@ int solve_linear_system(DATA *data, threadData_t *threadData, int sysNumber, dou
       throwStreamPrint(threadData, "OMC is compiled without UMFPACK, if you want use umfpack please compile OMC with UMFPACK.");
       break;
   #endif
-  #ifdef WITH_UMFPACK
+  #ifdef WITH_SUITESPARSE
     case LSS_KLU:
       success = solveKlu(data, threadData, sysNumber, aux_x);
       break;
@@ -580,7 +629,7 @@ int solve_linear_system(DATA *data, threadData_t *threadData, int sysNumber, dou
       success = solveLis(data, threadData, sysNumber, aux_x);
       break;
   #endif
-  #ifdef WITH_UMFPACK
+  #ifdef WITH_SUITESPARSE
     case LS_KLU:
       success = solveKlu(data, threadData, sysNumber, aux_x);
       break;
@@ -791,7 +840,7 @@ static void setBElementLis(int row, double value, void *data, threadData_t *thre
 }
 #endif
 
-#ifdef WITH_UMFPACK
+#ifdef WITH_SUITESPARSE
 static void setAElementUmfpack(int row, int col, double value, int nth, void *data, threadData_t *threadData)
 {
   LINEAR_SYSTEM_DATA* linSys = (LINEAR_SYSTEM_DATA*) data;
