@@ -56,11 +56,13 @@ typedef struct _FILE_INFO
 #define omc_dummyFileInfo {"",0,0,0,0,0}
 
 DLLExport extern void printInfo(FILE *stream, FILE_INFO info);
+// Defined in omc_error.c
 DLLExport extern void (*omc_assert)(threadData_t*, FILE_INFO, const char*, ...) __attribute__ ((noreturn));
 DLLExport extern void (*omc_assert_warning)(FILE_INFO, const char*, ...);
 DLLExport extern void (*omc_terminate)(FILE_INFO, const char*, ...);
 DLLExport extern void (*omc_throw)(threadData_t*) __attribute__ ((noreturn));
 
+// Defined in simulation_omc_assert.c
 DLLExport extern void (*omc_assert_withEquationIndexes)(threadData_t*,FILE_INFO, const int*, const char*, ...) __attribute__ ((noreturn));
 DLLExport extern void (*omc_assert_warning_withEquationIndexes)(FILE_INFO, const int*, const char*, ...);
 
@@ -84,6 +86,7 @@ enum LOG_STREAM
   LOG_DASSL,
   LOG_DASSL_STATES,
   LOG_DEBUG,
+  LOG_DELAY,
   LOG_DSS,
   LOG_DSS_JAC,
   LOG_DT,
