@@ -91,11 +91,6 @@ void alloc_real_array_data(real_array_t *a)
     a->data = real_alloc(base_array_nr_of_elements(*a));
 }
 
-void copy_real_array_data(const real_array_t source, real_array_t *dest)
-{
-    real_array_copy_data(source,*dest);
-}
-
 void copy_real_array_data_mem(const real_array_t source, modelica_real *dest)
 {
     size_t i, nr_of_elements;
@@ -309,6 +304,10 @@ void index_real_array(const real_array_t * source,
     omc_assert_macro(index_spec_ok(source_spec));
     omc_assert_macro(index_spec_fit_base_array(source_spec,source));
 
+    /* adrpo: if destination is an Real[0] just return */
+    if (dest->ndims == 1 && dest->dim_size[0] == 0)
+      return;
+
     /*for(i = 0, j = 0; i < source->ndims; ++i)
     {
       printf("source_spec->index_type[%d] = %c\n", i, source_spec->index_type[i]);
@@ -323,7 +322,7 @@ void index_real_array(const real_array_t * source,
             ++j;
         }
     }
-    omc_assert_macro(j == dest->ndims);
+    omc_assert_macro(imax(j,1) == dest->ndims);
 
     idx_vec1 = size_alloc(source->ndims);  /*indices in the source array*/
     /* idx_vec2 = size_alloc(dest->ndims); / * indices in the destination array* / */
@@ -334,7 +333,7 @@ void index_real_array(const real_array_t * source,
     }
     for(i = 0; i < source_spec->ndims; ++i) {
         if(source_spec->index[i] != NULL) { /* is 'S' or 'A' */
-            idx_size[i] = imax(source_spec->dim_size[i],1); /* the imax() is not needed, because there is (idx[d] >= size[d]) in the next_index(), but ... */
+            idx_size[i] = imax(source_spec->dim_size[i], 1); /* the imax() is not needed, because there is (idx[d] >= size[d]) in the next_index(), but ... */
         } else { /* is 'W' */
             idx_size[i] = source->dim_size[i];
         }
@@ -515,31 +514,6 @@ void array_alloc_scalar_real_array(real_array_t* dest, int n, modelica_real firs
     va_end(ap);
 }
 
-modelica_real* real_array_element_addr1(const real_array_t * source,int ndims,int dim1)
-{
-    return real_ptrget(source, dim1 - 1);
-}
-
-modelica_real* real_array_element_addr2(const real_array_t * source,int ndims,int dim1,int dim2)
-{
-    if (dim1 < 1 || dim1 > source->dim_size[0] || dim2 < 1 || dim2 > source->dim_size[1]) {
-      throwStreamPrint(NULL, "real_array_element_addr2: array has dimensions [%ld,%ld], got subscripts [%d,%d]", source->dim_size[1], source->dim_size[2], dim1, dim2);
-    }
-    return real_ptrget(source, ((dim1 - 1) * source->dim_size[1]) + (dim2 - 1));
-}
-
-modelica_real* real_array_element_addr(const real_array_t * source,int ndims,...)
-{
-    va_list ap;
-    modelica_real* tmp;
-
-    va_start(ap,ndims);
-    tmp = real_ptrget(source, calc_base_index_va(source, ndims, ap));
-    va_end(ap);
-
-    return tmp;
-}
-
 
 /* function: cat_real_array
  *
@@ -1086,7 +1060,7 @@ void exp_real_array(const real_array_t * a, modelica_integer n, real_array_t* de
     } else {
         if(n==1) {
             clone_real_array_spec(a,dest);
-            copy_real_array_data(*a,dest);
+            real_array_copy_data(*a, *dest);
         } else if (n==2) {
             clone_real_array_spec(a,dest);
             mul_real_matrix_product(a,a,dest);
@@ -1290,7 +1264,7 @@ void transpose_real_array(const real_array_t * a, real_array_t* dest)
     size_t n,m;
 
     if(a->ndims == 1) {
-        copy_real_array_data(*a,dest);
+        real_array_copy_data(*a, *dest);
         return;
     }
 
