@@ -163,50 +163,24 @@ void TextAnnotation::parseShapeAnnotation(QString annotation)
     return;
   }
   // 9th item of the list contains the extent points
-  QStringList extentsList = StringHandler::getStrings(StringHandler::removeFirstLastCurlBrackets(list.at(8)));
-  for (int i = 0 ; i < qMin(extentsList.size(), 2) ; i++) {
-    QStringList extentPoints = StringHandler::getStrings(StringHandler::removeFirstLastCurlBrackets(extentsList[i]));
-    if (extentPoints.size() >= 2)
-      mExtents.replace(i, QPointF(extentPoints.at(0).toFloat(), extentPoints.at(1).toFloat()));
-  }
+  mExtents.parse(list.at(8));
   // 10th item of the list contains the textString.
-  if (list.at(9).startsWith("{")) {
-    // DynamicSelect
-    QStringList args = StringHandler::getStrings(StringHandler::removeFirstLastCurlBrackets(list.at(9)));
-    if (args.count() > 0) {
-      mOriginalTextString = StringHandler::removeFirstLastQuotes(args.at(0));
-    }
-    if (args.count() > 1) {
-      mDynamicTextString << args.at(1);  // variable name
-    }
-    if (args.count() > 2) {
-      mDynamicTextString << args.at(2);  // significantDigits
-    }
-  } else {
-    mOriginalTextString = StringHandler::removeFirstLastQuotes(list.at(9));
-  }
-  mTextString = mOriginalTextString;
+  mTextString.parse(list.at(9));
   initUpdateTextString();
+
   // 11th item of the list contains the fontSize.
-  mFontSize = list.at(10).toFloat();
+  mFontSize.parse(list.at(10));
   // 12th item of the list contains the optional textColor, {-1, -1, -1} if not set
-  QStringList textColorList = StringHandler::getStrings(StringHandler::removeFirstLastCurlBrackets(list.at(11)));
-  if (textColorList.size() >= 3) {
-    int red, green, blue = 0;
-    red = textColorList.at(0).toInt();
-    green = textColorList.at(1).toInt();
-    blue = textColorList.at(2).toInt();
-    if (red >= 0 && green >= 0 && blue >= 0) {
-      mLineColor = QColor (red, green, blue);
-    }
+  if (!list.at(11).contains("-1")) {
+    mLineColor.parse(list.at(11));
   }
   // 13th item of the list contains the font name.
-  QString fontName = StringHandler::removeFirstLastQuotes(list.at(12));
+  QString fontName = StringHandler::removeFirstLastQuotes(stripDynamicSelect(list.at(12)));
   if (!fontName.isEmpty()) {
     mFontName = fontName;
   }
   // 14th item of the list contains the text styles.
-  QStringList textStyles = StringHandler::getStrings(StringHandler::removeFirstLastCurlBrackets(list.at(13)));
+  QStringList textStyles = StringHandler::getStrings(StringHandler::removeFirstLastCurlBrackets(stripDynamicSelect(list.at(13))));
   foreach (QString textStyle, textStyles) {
     if (textStyle == "TextStyle.Bold") {
       mTextStyles.append(StringHandler::TextStyleBold);
@@ -217,7 +191,7 @@ void TextAnnotation::parseShapeAnnotation(QString annotation)
     }
   }
   // 15th item of the list contains the text alignment.
-  QString horizontalAlignment = StringHandler::removeFirstLastQuotes(list.at(14));
+  QString horizontalAlignment = StringHandler::removeFirstLastQuotes(stripDynamicSelect(list.at(14)));
   if (horizontalAlignment == "TextAlignment.Left") {
     mHorizontalAlignment = StringHandler::TextAlignmentLeft;
   } else if (horizontalAlignment == "TextAlignment.Center") {
@@ -262,13 +236,13 @@ void TextAnnotation::paint(QPainter *painter, const QStyleOptionGraphicsItem *op
   Q_UNUSED(widget);
   //! @note We don't show text annotation that contains % for Library Icons or if it is too long.
   if (mpGraphicsView && mpGraphicsView->isRenderingLibraryPixmap()) {
-    if (mOriginalTextString.contains("%") || mOriginalTextString.length() > maxTextLengthToShowOnLibraryIcon) {
+    if (mTextString.contains("%") || mTextString.length() > maxTextLengthToShowOnLibraryIcon) {
       return;
     }
   } else if (mpComponent && mpComponent->getGraphicsView()->isRenderingLibraryPixmap()) {
     return;
   }
-  if (mVisible || !mDynamicVisible.isEmpty()) {
+  if (mVisible) {
     // state machine visualization
     // text annotation on a component
     if (mpComponent && mpComponent->getLibraryTreeItem() && mpComponent->getLibraryTreeItem()->isState()
@@ -289,20 +263,16 @@ void TextAnnotation::paint(QPainter *painter, const QStyleOptionGraphicsItem *op
         painter->setOpacity(0.2);
       }
     }
-    if (!mDynamicVisibleValue && ((mpGraphicsView && mpGraphicsView->isVisualizationView())
-                                  || (mpParentComponent && mpParentComponent->getGraphicsView()->isVisualizationView()))) {
-      return;
-    }
-    drawTextAnnotaion(painter);
+    drawTextAnnotation(painter);
   }
 }
 
 /*!
- * \brief TextAnnotation::drawTextAnnotaion
+ * \brief TextAnnotation::drawTextAnnotation
  * Draws the Text annotation
  * \param painter
  */
-void TextAnnotation::drawTextAnnotaion(QPainter *painter)
+void TextAnnotation::drawTextAnnotation(QPainter *painter)
 {
   applyLinePattern(painter);
   /* Don't apply the fill patterns on Text shapes. */
@@ -342,8 +312,8 @@ void TextAnnotation::drawTextAnnotaion(QPainter *painter)
   // map the existing bounding rect to new transformation but with positive width and height so that font metrics can work
   QRectF absMappedBoundingRect = QRectF(boundingRect().x() * sx, boundingRect().y() * sy, qAbs(boundingRect().width() * sx), qAbs(boundingRect().height() * sy));
   // normalize the text for drawing
-  mTextString = StringHandler::removeFirstLastQuotes(mTextString);
-  mTextString = StringHandler::unparse(QString("\"").append(mTextString).append("\""));
+  QString textString = StringHandler::removeFirstLastQuotes(mTextString);
+  textString = StringHandler::unparse(QString("\"").append(mTextString).append("\""));
   // Don't create new QFont instead get a font from painter and set the values on it and set it back.
   QFont font = painter->font();
   font.setFamily(mFontName);
@@ -363,7 +333,7 @@ void TextAnnotation::drawTextAnnotaion(QPainter *painter)
   // if absolute font size is defined and is greater than 0 then we don't need to calculate the font size.
   if (mFontSize <= 0) {
     QFontMetrics fontMetrics(painter->font());
-    QRect fontBoundRect = fontMetrics.boundingRect(absMappedBoundingRect.toRect(), Qt::TextDontClip, mTextString);
+    QRect fontBoundRect = fontMetrics.boundingRect(absMappedBoundingRect.toRect(), Qt::TextDontClip, textString);
     const qreal xFactor = absMappedBoundingRect.width() / fontBoundRect.width();
     const qreal yFactor = absMappedBoundingRect.height() / fontBoundRect.height();
     /* Ticket:4256
@@ -379,10 +349,10 @@ void TextAnnotation::drawTextAnnotaion(QPainter *painter)
   /* Try to get the elided text if calculated font size <= Helper::minimumTextFontSize
    * OR if font size is absolute.
    */
-  QString textToDraw = mTextString;
+  QString textToDraw = textString;
   if (absMappedBoundingRect.width() > 1 && ((mFontSize <= 0 && painter->font().pointSizeF() <= Helper::minimumTextFontSize) || mFontSize > 0)) {
     QFontMetrics fontMetrics(painter->font());
-    textToDraw = fontMetrics.elidedText(mTextString, Qt::ElideRight, absMappedBoundingRect.width());
+    textToDraw = fontMetrics.elidedText(textString, Qt::ElideRight, absMappedBoundingRect.width());
     // if we get "..." i.e., QChar(0x2026) as textToDraw then don't draw anything
     if (textToDraw.compare(QChar(0x2026)) == 0) {
       textToDraw = "";
@@ -404,26 +374,13 @@ QString TextAnnotation::getOMCShapeAnnotation()
   annotationString.append(GraphicItem::getOMCShapeAnnotation());
   annotationString.append(FilledShape::getOMCShapeAnnotation());
   // get the extents
-  QString extentString;
-  extentString.append("{");
-  extentString.append("{").append(QString::number(mExtents.at(0).x())).append(",");
-  extentString.append(QString::number(mExtents.at(0).y())).append("},");
-  extentString.append("{").append(QString::number(mExtents.at(1).x())).append(",");
-  extentString.append(QString::number(mExtents.at(1).y())).append("}");
-  extentString.append("}");
-  annotationString.append(extentString);
+  annotationString.append(mExtents.toQString());
   // get the text string
-  annotationString.append(QString("\"").append(mOriginalTextString).append("\""));
+  annotationString.append(mTextString.toQString());
   // get the font size
-  annotationString.append(QString::number(mFontSize));
+  annotationString.append(mFontSize.toQString());
   // get the text color
-  QString textColorString;
-  textColorString.append("{");
-  textColorString.append(QString::number(mLineColor.red())).append(",");
-  textColorString.append(QString::number(mLineColor.green())).append(",");
-  textColorString.append(QString::number(mLineColor.blue()));
-  textColorString.append("}");
-  annotationString.append(textColorString);
+  annotationString.append(mLineColor.toQString());
   // get the font name
   if (!mFontName.isEmpty() && mFontName.compare(Helper::systemFontInfo.family()) != 0) {
     annotationString.append(QString("\"").append(mFontName).append("\""));
@@ -462,23 +419,16 @@ QString TextAnnotation::getShapeAnnotation()
 {
   QStringList annotationString;
   annotationString.append(GraphicItem::getShapeAnnotation());
-  annotationString.append(FilledShape::getShapeAnnotation());
+  annotationString.append(FilledShape::getTextShapeAnnotation());
   // get the extents
-  if (mExtents.size() > 1) {
-    QString extentString;
-    extentString.append("extent={");
-    extentString.append("{").append(QString::number(mExtents.at(0).x())).append(",");
-    extentString.append(QString::number(mExtents.at(0).y())).append("},");
-    extentString.append("{").append(QString::number(mExtents.at(1).x())).append(",");
-    extentString.append(QString::number(mExtents.at(1).y())).append("}");
-    extentString.append("}");
-    annotationString.append(extentString);
+  if (mExtents.isDynamicSelectExpression() || mExtents.size() > 1) {
+    annotationString.append(QString("extent=%1").arg(mExtents.toQString()));
   }
   // get the text string
-  annotationString.append(QString("textString=\"").append(mOriginalTextString).append("\""));
+  annotationString.append(QString("textString=%1").arg(mTextString.toQString()));
   // get the font size
-  if (mFontSize != 0) {
-    annotationString.append(QString("fontSize=").append(QString::number(mFontSize)));
+  if (mFontSize.isDynamicSelectExpression() || mFontSize != 0) {
+    annotationString.append(QString("fontSize=%1").arg(mFontSize.toQString()));
   }
   // get the font name
   /* Ticket:4204
@@ -519,7 +469,7 @@ void TextAnnotation::updateShape(ShapeAnnotation *pShapeAnnotation)
 void TextAnnotation::initUpdateTextString()
 {
   if (mpComponent) {
-    if (mOriginalTextString.contains("%")) {
+    if (mTextString.contains("%")) {
       updateTextString();
       connect(mpComponent, SIGNAL(displayTextChanged()), SLOT(updateTextString()), Qt::UniqueConnection);
     }
@@ -538,6 +488,7 @@ void TextAnnotation::updateTextStringHelper(QRegExp regExp)
     QString variable = regExp.cap(0).trimmed();
     if ((!variable.isEmpty()) && (variable.compare("%%") != 0) && (variable.compare("%name") != 0) && (variable.compare("%class") != 0)) {
       variable.remove("%");
+      variable = StringHandler::removeFirstLastCurlBrackets(variable);
       if (!variable.isEmpty()) {
         QString textValue;
         /* Ticket:4204
@@ -545,21 +496,37 @@ void TextAnnotation::updateTextStringHelper(QRegExp regExp)
          */
         textValue = mpComponent->getRootParentComponent()->getParameterDisplayString(variable);
         if (!textValue.isEmpty()) {
-          QString unit = "";
-          QString displaytUnit = "";
-          Element *pComponent = mpComponent->getRootParentComponent()->getComponentByName(variable);
-          if (pComponent) {
-            displaytUnit = pComponent->getDerivedClassModifierValue("displaytUnit");
-            if (displaytUnit.isEmpty()) {
-              unit = pComponent->getDerivedClassModifierValue("unit");
-              displaytUnit = unit;
+          QString unit = mpComponent->getRootParentComponent()->getParameterModifierValue(variable, "unit");
+          QString displayUnit = mpComponent->getRootParentComponent()->getParameterModifierValue(variable, "displayUnit");
+          Element *pElement = mpComponent->getRootParentComponent()->getElementByName(variable);
+          if (pElement) {
+            if (displayUnit.isEmpty()) {
+              displayUnit = pElement->getDerivedClassModifierValue("displayUnit");
+            }
+            if (unit.isEmpty()) {
+              unit = pElement->getDerivedClassModifierValue("unit");
+            }
+            // if display unit is still empty then use unit
+            if (displayUnit.isEmpty()) {
+              displayUnit = unit;
             }
           }
-          if (displaytUnit.isEmpty()) {
+          if (displayUnit.isEmpty() || unit.isEmpty()) {
             mTextString.replace(pos, regExp.matchedLength(), textValue);
             pos += textValue.length();
           } else {
-            QString textValueWithDisplayUnit = QString("%1 %2").arg(textValue, displaytUnit);
+            QString textValueWithDisplayUnit;
+            OMCProxy *pOMCProxy = MainWindow::instance()->getOMCProxy();
+            OMCInterface::convertUnits_res convertUnit = pOMCProxy->convertUnits(unit, displayUnit);
+            if (convertUnit.unitsCompatible) {
+              qreal convertedValue = Utilities::convertUnit(textValue.toDouble(), convertUnit.offset, convertUnit.scaleFactor);
+              textValue = StringHandler::number(convertedValue);
+              displayUnit = Utilities::convertUnitToSymbol(displayUnit);
+              textValueWithDisplayUnit = QString("%1 %2").arg(textValue, displayUnit);
+            } else {
+              unit = Utilities::convertUnitToSymbol(unit);
+              textValueWithDisplayUnit = QString("%1 %2").arg(textValue, unit);
+            }
             mTextString.replace(pos, regExp.matchedLength(), textValueWithDisplayUnit);
             pos += textValueWithDisplayUnit.length();
           }
@@ -593,7 +560,7 @@ void TextAnnotation::updateTextString()
    */
   LineAnnotation *pLineAnnotation = dynamic_cast<LineAnnotation*>(parentItem());
   if (pLineAnnotation) {
-    if (mOriginalTextString.toLower().contains("%condition")) {
+    if (mTextString.toLower().contains("%condition")) {
       if (!pLineAnnotation->getCondition().isEmpty()) {
         mTextString.replace(QRegExp("%condition"), pLineAnnotation->getCondition());
       }
@@ -602,25 +569,26 @@ void TextAnnotation::updateTextString()
       }
     }
   } else if (mpComponent) {
-    mTextString = mOriginalTextString;
+    mTextString.reset();
+
     if (!mTextString.contains("%")) {
       return;
     }
-    if (mOriginalTextString.toLower().contains("%name")) {
+    if (mTextString.toLower().contains("%name")) {
       mTextString.replace(QRegExp("%name"), mpComponent->getName());
     }
-    if (mOriginalTextString.toLower().contains("%class") && mpComponent->getLibraryTreeItem()) {
+    if (mTextString.toLower().contains("%class") && mpComponent->getLibraryTreeItem()) {
       mTextString.replace(QRegExp("%class"), mpComponent->getLibraryTreeItem()->getNameStructure());
     }
     if (!mTextString.contains("%")) {
       return;
     }
     /* handle variables now */
-    updateTextStringHelper(QRegExp("(%%|%\\w*)"));
+    updateTextStringHelper(QRegExp("(%%|%\\{?\\w+(\\.\\w+)*\\}?)"));
     /* call again with non-word characters so invalid % can be removed. */
-    updateTextStringHelper(QRegExp("(%%|%\\W*)"));
+    updateTextStringHelper(QRegExp("(%%|%\\{?\\W+(\\.\\W+)*\\}?)"));
     /* handle %% */
-    if (mOriginalTextString.toLower().contains("%%")) {
+    if (mTextString.toLower().contains("%%")) {
       mTextString.replace(QRegExp("%%"), "%");
     }
   }
