@@ -37,6 +37,7 @@
 #include "Modeling/MessagesWidget.h"
 #include "Modeling/Commands.h"
 #include "Options/OptionsDialog.h"
+#include "OMPlot.h"
 
 #include <QApplication>
 #include <QMenu>
@@ -101,22 +102,40 @@ Parameter::Parameter(Element *pComponent, bool showStartAttribute, QString tab,
   setSaveSelectorCaption("-");
   createValueWidget();
   // Get unit value
-  QString unit = mpComponent->getDerivedClassModifierValue("unit");
-  mUnit = unit;
+  mUnit = mpComponent->getDerivedClassModifierValue("unit");;
   // Get displayUnit value
   QString displayUnit = mpComponent->getDerivedClassModifierValue("displayUnit");
   if (displayUnit.isEmpty()) {
-    displayUnit = unit;
+    displayUnit = mUnit;
   }
   mDisplayUnit = StringHandler::removeFirstLastQuotes(displayUnit);
   mPreviousUnit = mDisplayUnit;
-  mpUnitComboBox = new QComboBox;
+  QStringList units;
   if (!mUnit.isEmpty()) {
-    mpUnitComboBox->addItem(Utilities::convertUnitToSymbol(mUnit), mUnit);
+    units << mUnit;
     if (mDisplayUnit.compare(mUnit) != 0) {
-      mpUnitComboBox->addItem(Utilities::convertUnitToSymbol(mDisplayUnit), mDisplayUnit);
-      mpUnitComboBox->setCurrentIndex(1);
+      units << mDisplayUnit;
     }
+    Utilities::addDefaultDisplayUnit(mUnit, units);
+    // add unit prefixes
+    if (OMPlot::Plot::prefixableUnit(mUnit)) {
+      units << QString("k%1").arg(mUnit)
+            << QString("M%1").arg(mUnit)
+            << QString("G%1").arg(mUnit)
+            << QString("T%1").arg(mUnit)
+            << QString("m%1").arg(mUnit)
+            << QString("u%1").arg(mUnit)
+            << QString("n%1").arg(mUnit)
+            << QString("p%1").arg(mUnit);
+    }
+  }
+  mpUnitComboBox = new QComboBox;
+  units.removeDuplicates();
+  foreach (QString unit, units) {
+    mpUnitComboBox->addItem(Utilities::convertUnitToSymbol(unit), unit);
+  }
+  if (mDisplayUnit.compare(mUnit) != 0) {
+    mpUnitComboBox->setCurrentIndex(1);
   }
   connect(mpUnitComboBox, SIGNAL(currentIndexChanged(int)), SLOT(unitComboBoxChanged(int)));
   mpCommentLabel = new Label(mpComponent->getComponentInfo()->getComment());
@@ -371,9 +390,22 @@ void Parameter::createValueWidget()
         constrainedByClassName = className;
       }
 
+      // add choices if there are any
+      if (mpComponent->hasChoices())
+      {
+          QStringList choices = mpComponent->getChoices();
+          for (i = 0; i < choices.size(); i++) {
+            QString choice = choices[i];
+            QString comment = StringHandler::getModelicaComment(choice);
+            mpValueComboBox->addItem(comment, choice);
+          }
+      }
+
+      // do replaceable only if not choicesAllMatching=false
+      // if choicesAllMatching is not defined, consider choicesAllMatching=true
       parentClassName = mpComponent->getComponentInfo()->getParentClassName();
       replaceableChoices = pOMCProxy->getAllSubtypeOf(constrainedByClassName, parentClassName);
-      for (i = 0 ; i < replaceableChoices.size(); i++) {
+      for (i = 0; i < replaceableChoices.size(); i++) {
         replaceableChoice = replaceableChoices[i];
         // if replaceableChoices points to a class in this scope, remove scope
         if (replaceableChoice.startsWith(parentClassName + "."))
@@ -412,6 +444,10 @@ void Parameter::createValueWidget()
  */
 void Parameter::enableDisableUnitComboBox(const QString &value)
 {
+  // Do not do anything when the value is empty.
+  if (value.isEmpty()) {
+    return;
+  }
   /* Enable/disable the unit combobox based on the literalConstant
    * Set the display unit as current when value is literalConstant otherwise use unit
    * ticket:5618 Disable the unit drop down when we have a symbolic parameter
@@ -423,7 +459,7 @@ void Parameter::enableDisableUnitComboBox(const QString &value)
    */
   if (!literalConstant) {
     bool state = mpUnitComboBox->blockSignals(true);
-    int index = mpUnitComboBox->findData(literalConstant ? mDisplayUnit : mUnit);
+    int index = mpUnitComboBox->findData(mUnit);
     if (index > -1 && index != mpUnitComboBox->currentIndex()) {
       mpUnitComboBox->setCurrentIndex(index);
       mPreviousUnit = mpUnitComboBox->itemData(mpUnitComboBox->currentIndex()).toString();
@@ -1383,21 +1419,20 @@ void ElementParameters::updateComponentParameters()
 
 /*!
  * \class ElementAttributes
- * \brief A dialog for displaying components attributes like visibility, stream, casuality etc.
+ * \brief A dialog for displaying elements attributes like visibility, stream, casuality etc.
  */
 /*!
  * \brief ElementAttributes::ElementAttributes
- * \param pComponent
+ * \param pElement
  * \param pParent
  */
-ElementAttributes::ElementAttributes(Element *pComponent, QWidget *pParent)
+ElementAttributes::ElementAttributes(Element *pElement, QWidget *pParent)
   : QDialog(pParent)
 {
-  QString className = pComponent->getGraphicsView()->getModelWidget()->getLibraryTreeItem()->getNameStructure();
-  setWindowTitle(tr("%1 - %2 - %3 in %4").arg(Helper::applicationName).arg(tr("Element Attributes")).arg(pComponent->getName())
-                 .arg(className));
+  QString className = pElement->getGraphicsView()->getModelWidget()->getLibraryTreeItem()->getNameStructure();
+  setWindowTitle(tr("%1 - %2 - %3 in %4").arg(Helper::applicationName, tr("Element Attributes"), pElement->getName(), className));
   setAttribute(Qt::WA_DeleteOnClose);
-  mpComponent = pComponent;
+  mpElement = pElement;
   setUpDialog();
   initializeDialog();
 }
@@ -1486,13 +1521,13 @@ void ElementAttributes::setUpDialog()
   // Create the buttons
   mpOkButton = new QPushButton(Helper::ok);
   mpOkButton->setAutoDefault(true);
-  connect(mpOkButton, SIGNAL(clicked()), this, SLOT(updateComponentAttributes()));
+  connect(mpOkButton, SIGNAL(clicked()), this, SLOT(updateElementAttributes()));
   mpCancelButton = new QPushButton(Helper::cancel);
   mpCancelButton->setAutoDefault(false);
   connect(mpCancelButton, SIGNAL(clicked()), this, SLOT(reject()));
   mpButtonBox = new QDialogButtonBox(Qt::Horizontal);
   mpButtonBox->addButton(mpOkButton, QDialogButtonBox::ActionRole);
-  if (mpComponent->getGraphicsView()->getModelWidget()->getLibraryTreeItem()->isSystemLibrary() || mpComponent->isInheritedComponent()) {
+  if (mpElement->getGraphicsView()->getModelWidget()->getLibraryTreeItem()->isSystemLibrary() || mpElement->isInheritedComponent()) {
     mpOkButton->setDisabled(true);
   }
   mpButtonBox->addButton(mpCancelButton, QDialogButtonBox::ActionRole);
@@ -1517,79 +1552,78 @@ void ElementAttributes::setUpDialog()
 void ElementAttributes::initializeDialog()
 {
   // get Class Name
-  mpNameTextBox->setText(mpComponent->getComponentInfo()->getName());
+  mpNameTextBox->setText(mpElement->getComponentInfo()->getName());
   mpNameTextBox->setCursorPosition(0);
   // get dimensions
-  QString dimensions = mpComponent->getComponentInfo()->getArrayIndex();
+  QString dimensions = mpElement->getComponentInfo()->getArrayIndex();
   mpDimensionsTextBox->setText(QString("[%1]").arg(dimensions));
   // get Comment
-  mpCommentTextBox->setText(mpComponent->getComponentInfo()->getComment());
+  mpCommentTextBox->setText(mpElement->getComponentInfo()->getComment());
   mpCommentTextBox->setCursorPosition(0);
   // get classname
-  mpPathTextBox->setText(mpComponent->getComponentInfo()->getClassName());
+  mpPathTextBox->setText(mpElement->getComponentInfo()->getClassName());
   // get Variability
-  if (mpComponent->getComponentInfo()->getVariablity() == "constant") {
+  if (mpElement->getComponentInfo()->getVariablity() == "constant") {
     mpConstantRadio->setChecked(true);
-  } else if (mpComponent->getComponentInfo()->getVariablity() == "parameter") {
+  } else if (mpElement->getComponentInfo()->getVariablity() == "parameter") {
     mpParameterRadio->setChecked(true);
-  } else if (mpComponent->getComponentInfo()->getVariablity() == "discrete") {
+  } else if (mpElement->getComponentInfo()->getVariablity() == "discrete") {
     mpDiscreteRadio->setChecked(true);
   } else {
     mpDefaultRadio->setChecked(true);
   }
   // get Properties
-  mpFinalCheckBox->setChecked(mpComponent->getComponentInfo()->getFinal());
-  mpProtectedCheckBox->setChecked(mpComponent->getComponentInfo()->getProtected());
-  mpReplaceAbleCheckBox->setChecked(mpComponent->getComponentInfo()->getReplaceable());
-  mIsFlow = mpComponent->getComponentInfo()->getFlow() ? "true" : "false";
+  mpFinalCheckBox->setChecked(mpElement->getComponentInfo()->getFinal());
+  mpProtectedCheckBox->setChecked(mpElement->getComponentInfo()->getProtected());
+  mpReplaceAbleCheckBox->setChecked(mpElement->getComponentInfo()->getReplaceable());
+  mIsFlow = mpElement->getComponentInfo()->getFlow() ? "true" : "false";
   // get Casuality
-  if (mpComponent->getComponentInfo()->getCausality() == "input") {
+  if (mpElement->getComponentInfo()->getCausality() == "input") {
     mpInputRadio->setChecked(true);
-  } else if (mpComponent->getComponentInfo()->getCausality() == "output") {
+  } else if (mpElement->getComponentInfo()->getCausality() == "output") {
     mpOutputRadio->setChecked(true);
   } else {
     mpNoneRadio->setChecked(true);
   }
   // get InnerOuter
-  mpInnerCheckBox->setChecked(mpComponent->getComponentInfo()->getInner());
-  mpOuterCheckBox->setChecked(mpComponent->getComponentInfo()->getOuter());
+  mpInnerCheckBox->setChecked(mpElement->getComponentInfo()->getInner());
+  mpOuterCheckBox->setChecked(mpElement->getComponentInfo()->getOuter());
 }
 
 /*!
- * \brief ElementAttributes::updateComponentAttributes
+ * \brief ElementAttributes::updateElementAttributes
  * Slot activated when mpOkButton clicked signal is raised.\n
- * Updates the component attributes.
+ * Updates the element attributes.
  */
-void ElementAttributes::updateComponentAttributes()
+void ElementAttributes::updateElementAttributes()
 {
-  ModelWidget *pModelWidget = mpComponent->getGraphicsView()->getModelWidget();
+  ModelWidget *pModelWidget = mpElement->getGraphicsView()->getModelWidget();
   /* Check the same component name problem before setting any attributes. */
-  if (mpComponent->getComponentInfo()->getName().compare(mpNameTextBox->text()) != 0) {
-    if (!mpComponent->getGraphicsView()->checkElementName(mpNameTextBox->text())) {
-      QMessageBox::information(MainWindow::instance(), QString(Helper::applicationName).append(" - ").append(Helper::information),
+  if (mpElement->getComponentInfo()->getName().compare(mpNameTextBox->text()) != 0) {
+    if (!mpElement->getGraphicsView()->checkElementName(mpNameTextBox->text())) {
+      QMessageBox::information(MainWindow::instance(), QString("%1 - %2").arg(Helper::applicationName, Helper::information),
                                GUIMessages::getMessage(GUIMessages::SAME_COMPONENT_NAME).arg(mpNameTextBox->text()), Helper::ok);
       return;
     }
   }
   // check for spaces
   if (StringHandler::containsSpace(mpNameTextBox->text())) {
-    QMessageBox::critical(MainWindow::instance(), QString("%1 - %2").arg(Helper::applicationName).arg(Helper::error),
+    QMessageBox::critical(MainWindow::instance(), QString("%1 - %2").arg(Helper::applicationName, Helper::error),
                           tr("A component name should not have spaces. Please choose another name."), Helper::ok);
     return;
   }
   // check for comma
   if (mpNameTextBox->text().contains(',')) {
-    QMessageBox::critical(MainWindow::instance(), QString("%1 - %2").arg(Helper::applicationName).arg(Helper::error),
+    QMessageBox::critical(MainWindow::instance(), QString("%1 - %2").arg(Helper::applicationName, Helper::error),
                           GUIMessages::getMessage(GUIMessages::INVALID_INSTANCE_NAME).arg(mpNameTextBox->text()), Helper::ok);
     return;
   }
   // check for invalid names
   MainWindow::instance()->getOMCProxy()->setLoggingEnabled(false);
-  QList<QString> result = MainWindow::instance()->getOMCProxy()->parseString(QString("model M N %1; end M;").arg(mpNameTextBox->text()),
-                                                                             "M", false);
+  QList<QString> result = MainWindow::instance()->getOMCProxy()->parseString(QString("model M N %1; end M;").arg(mpNameTextBox->text()), "M", false);
   MainWindow::instance()->getOMCProxy()->setLoggingEnabled(true);
   if (result.isEmpty()) {
-    QMessageBox::critical(MainWindow::instance(), QString("%1 - %2").arg(Helper::applicationName).arg(Helper::error),
+    QMessageBox::critical(MainWindow::instance(), QString("%1 - %2").arg(Helper::applicationName, Helper::error),
                           GUIMessages::getMessage(GUIMessages::INVALID_INSTANCE_NAME).arg(mpNameTextBox->text()), Helper::ok);
     return;
   }
@@ -1612,16 +1646,16 @@ void ElementAttributes::updateComponentAttributes()
     causality = "";
   }
   // save the old ElementInfo
-  ElementInfo oldComponentInfo(mpComponent->getComponentInfo());
+  ElementInfo oldComponentInfo(mpElement->getComponentInfo());
   // Create a new ElementInfo
   ElementInfo newComponentInfo;
-  newComponentInfo.setClassName(mpComponent->getComponentInfo()->getClassName());
+  newComponentInfo.setClassName(mpElement->getComponentInfo()->getClassName());
   newComponentInfo.setName(mpNameTextBox->text());
   newComponentInfo.setComment(mpCommentTextBox->text());
   newComponentInfo.setProtected(mpProtectedCheckBox->isChecked());
   newComponentInfo.setFinal(mpFinalCheckBox->isChecked());
-  newComponentInfo.setFlow(mpComponent->getComponentInfo()->getFlow());
-  newComponentInfo.setStream(mpComponent->getComponentInfo()->getStream());
+  newComponentInfo.setFlow(mpElement->getComponentInfo()->getFlow());
+  newComponentInfo.setStream(mpElement->getComponentInfo()->getStream());
   newComponentInfo.setReplaceable(mpReplaceAbleCheckBox->isChecked());
   newComponentInfo.setVariablity(variability);
   newComponentInfo.setInner(mpInnerCheckBox->isChecked());
@@ -1632,7 +1666,7 @@ void ElementAttributes::updateComponentAttributes()
   /* If user has really changed the Component's attributes then push that change on the stack.
    */
   if (oldComponentInfo != newComponentInfo) {
-    UpdateComponentAttributesCommand *pUpdateComponentAttributesCommand = new UpdateComponentAttributesCommand(mpComponent, oldComponentInfo, newComponentInfo);
+    UpdateComponentAttributesCommand *pUpdateComponentAttributesCommand = new UpdateComponentAttributesCommand(mpElement, oldComponentInfo, newComponentInfo);
     pModelWidget->getUndoStack()->push(pUpdateComponentAttributesCommand);
     pModelWidget->updateModelText();
   }
