@@ -43,6 +43,9 @@
 #include <fmilib.h>
 #include <JM/jm_portability.h>
 #include <RegEx.h>
+#include <unordered_set>
+#include <cmath>
+
 
 oms::ComponentFMUCS::ComponentFMUCS(const ComRef& cref, System* parentSystem, const std::string& fmuPath)
   : oms::Component(cref, oms_component_fmu, parentSystem, fmuPath), fmuInfo(fmuPath)
@@ -140,64 +143,59 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const oms::ComRef& cref, oms::
   // create a list of all variables
   fmi2_import_variable_list_t *varList = fmi2_import_get_variable_list(component->fmu, 0);
   size_t varListSize = fmi2_import_get_variable_list_size(varList);
-  logDebug(std::to_string(varListSize) + " variables");
   component->allVariables.reserve(varListSize);
   component->exportVariables.reserve(varListSize);
   for (size_t i = 0; i < varListSize; ++i)
   {
     fmi2_import_variable_t* var = fmi2_import_get_variable(varList, i);
-    oms::Variable v(var, i + 1);
+    oms::Variable v(var);
+    if (v.getIndex() != i)
+    {
+      logError("Index mismatch " + std::to_string(v.getIndex()) + " != " + std::to_string(i) + ".\nPlease report the problem to the dev team: https://github.com/OpenModelica/OMSimulator/issues/new?assignees=&labels=&template=bug_report.md");
+      fmi2_import_free_variable_list(varList);
+      delete component;
+      return NULL;
+    }
     component->allVariables.push_back(v);
     component->exportVariables.push_back(true);
   }
   fmi2_import_free_variable_list(varList);
 
-  // mark states
+  // extract continuous-time derivatives
   varList = fmi2_import_get_derivatives_list(component->fmu);
   varListSize = fmi2_import_get_variable_list_size(varList);
-  logDebug(std::to_string(varListSize) + " states");
   for (size_t i = 0; i < varListSize; ++i)
   {
     fmi2_import_variable_t* var = fmi2_import_get_variable(varList, i);
-    fmi2_import_real_variable_t* varReal = fmi2_import_get_variable_as_real(var);
-    fmi2_import_variable_t* varState = (fmi2_import_variable_t*)fmi2_import_get_real_variable_derivative_of(varReal);
-    if (varState)
-    {
-      // IMPORTANT: vr is not unique!!! Do lookup with proper index or name
-      size_t originalIndex = fmi2_import_get_variable_original_order(varState);
-      if (originalIndex < 0 || originalIndex >= component->allVariables.size())
-      {
-        logError("Couldn't find " + std::string(fmi2_import_get_variable_name(varState)));
-        fmi2_import_free_variable_list(varList);
-        delete component;
-        return NULL;
-      }
-      component->allVariables[originalIndex].markAsState();
-    }
-    else
-    {
-      logError("Couldn't map " + std::string(fmi2_import_get_variable_name(var)) + " to the corresponding state variable");
-      fmi2_import_free_variable_list(varList);
-      delete component;
-      return NULL;
-    }
+    size_t originalIndex = fmi2_import_get_variable_original_order(var);
+    component->allVariables[originalIndex].markAsContinuousTimeDer();
+    component->derivatives.push_back(originalIndex);
   }
   fmi2_import_free_variable_list(varList);
 
+  // mark states and continuous-time states
+  for (size_t i = 0; i < component->allVariables.size(); ++i)
+  {
+    if (component->allVariables[i].isContinuousTimeDer())
+      component->allVariables[component->allVariables[i].getStateIndex()].markAsContinuousTimeState(i);
+    else if (component->allVariables[i].isDer())
+      component->allVariables[component->allVariables[i].getStateIndex()].markAsState(i);
+  }
+
   // create some special variable maps
   for (auto const& v : component->allVariables)
   {
     if (v.isInput())
-      component->inputs.push_back(v);
+      component->inputs.push_back(v.getIndex());
     else if (v.isOutput())
     {
-      component->outputs.push_back(v);
+      component->outputs.push_back(v.getIndex());
       component->outputsGraph.addNode(Connector(oms_causality_output, v.getType(), v.getCref(), component->getFullCref()));
     }
     else if (v.isParameter())
-      component->parameters.push_back(v);
+      component->parameters.push_back(v.getIndex());
     else if (v.isCalculatedParameter())
-      component->calculatedParameters.push_back(v);
+      component->calculatedParameters.push_back(v.getIndex());
 
     if (v.isInitialUnknown())
       component->initialUnknownsGraph.addNode(Connector(v.getCausality(), v.getType(), v.getCref(), component->getFullCref()));
@@ -209,18 +207,18 @@ oms::Component* oms::ComponentFMUCS::NewComponent(const oms::ComRef& cref, oms::
   while (component->connectors.size() > 0 && NULL == component->connectors.back())
     component->connectors.pop_back();
 
-  int i = 1;
+  int j = 1;
   int size = 1 + component->inputs.size();
-  for (const auto& v : component->inputs)
-    component->connectors.push_back(new Connector(oms_causality_input, v.getType(), v.getCref(), component->getFullCref(), i++/(double)size));
-  i = 1;
+  for (const auto& i : component->inputs)
+    component->connectors.push_back(new Connector(oms_causality_input, component->allVariables[i].getType(), component->allVariables[i].getCref(), component->getFullCref(), j++/(double)size));
+  j = 1;
   size = 1 + component->outputs.size();
-  for (const auto& v : component->outputs)
-    component->connectors.push_back(new Connector(oms_causality_output, v.getType(), v.getCref(), component->getFullCref(), i++/(double)size));
-  for (const auto& v : component->parameters)
-    component->connectors.push_back(new Connector(oms_causality_parameter, v.getType(), v.getCref(), component->getFullCref()));
-  for (const auto& v : component->calculatedParameters)
-    component->connectors.push_back(new Connector(oms_causality_calculatedParameter, v.getType(), v.getCref(), component->getFullCref()));
+  for (const auto& i : component->outputs)
+    component->connectors.push_back(new Connector(oms_causality_output, component->allVariables[i].getType(), component->allVariables[i].getCref(), component->getFullCref(), j++/(double)size));
+  for (const auto& i : component->parameters)
+    component->connectors.push_back(new Connector(oms_causality_parameter, component->allVariables[i].getType(), component->allVariables[i].getCref(), component->getFullCref()));
+  for (const auto& i : component->calculatedParameters)
+    component->connectors.push_back(new Connector(oms_causality_calculatedParameter, component->allVariables[i].getType(), component->allVariables[i].getCref(), component->getFullCref()));
   component->connectors.push_back(NULL);
   component->element.setConnectors(&component->connectors[0]);
 
@@ -335,14 +333,20 @@ oms_status_enu_t oms::ComponentFMUCS::exportToSSD(pugi::xml_node& node, Snapshot
   return oms_status_ok;
 }
 
+void oms::ComponentFMUCS::getFilteredUnitDefinitionsToSSD(std::map<std::string, std::map<std::string, std::string>>& unitDefinitions)
+{
+  return values.getFilteredUnitDefinitionsToSSD(unitDefinitions);
+}
+
 oms_status_enu_t oms::ComponentFMUCS::exportToSSV(pugi::xml_node& ssvNode)
 {
   return values.exportToSSV(ssvNode);
 }
 
-oms_status_enu_t oms::ComponentFMUCS::exportToSSVTemplate(pugi::xml_node& ssvNode)
+oms_status_enu_t oms::ComponentFMUCS::exportToSSVTemplate(pugi::xml_node& ssvNode, Snapshot& snapshot)
 {
   values.exportToSSVTemplate(ssvNode, getCref());
+  values.exportUnitDefinitionsToSSVTemplate(snapshot, "template.ssv");
   return oms_status_ok;
 }
 
@@ -352,61 +356,93 @@ oms_status_enu_t oms::ComponentFMUCS::exportToSSMTemplate(pugi::xml_node& ssmNod
   return oms_status_ok;
 }
 
+void oms::ComponentFMUCS::dumpInitialUnknowns()
+{
+  std::string str = "";
+  int n=0;
+  for (auto &v : allVariables)
+  {
+    if (v.isInitialUnknown())
+    {
+      n++;
+      if (!str.empty())
+        str += ", ";
+      str += std::to_string(v.getIndex()+1) + ": " + std::string(v.getCref());
+    }
+  }
+  logInfo("[" + std::string(getCref()) + ": " + getPath() + "] The FMU contains " + std::to_string(n) + " initial unknowns: " + str);
+}
+
 oms_status_enu_t oms::ComponentFMUCS::initializeDependencyGraph_initialUnknowns()
 {
-  if (initialUnknownsGraph.getEdges().size() > 0)
+  if (initialUnknownsGraph.getEdges().connections.size() > 0)
     return logError(std::string(getCref()) + ": " + getPath() + " is already initialized");
 
-  int N=initialUnknownsGraph.getNodes().size();
+  //dumpInitialUnknowns();
 
   // Check if initial unknowns from modelDescription.xml are the same as in initialUnknownsGraph
-  fmi2_import_variable_list_t* initialUnknowns;
-  initialUnknowns = fmi2_import_get_initial_unknowns_list(fmu);
-  size_t N_fmilib = fmi2_import_get_variable_list_size(initialUnknowns);
+  fmi2_import_variable_list_t* initialUnknowns_ModelStructure;
+  initialUnknowns_ModelStructure = fmi2_import_get_initial_unknowns_list(fmu);
+  size_t N_ModelStructure = fmi2_import_get_variable_list_size(initialUnknowns_ModelStructure);
+  size_t N = initialUnknownsGraph.getNodes().size();
+
   bool badInitialUnknowns = false;
 
-  if (N != N_fmilib)
+  std::unordered_set <unsigned int> setA_ModelStructure;
+  for (size_t i=0; i < N_ModelStructure; ++i)
   {
-    if (!Flags::IgnoreInitialUnknowns())
-      logWarning("[" + std::string(getCref()) + ": " + getPath() + "] The FMU lists " + std::to_string(N_fmilib) + " initial unknowns but actually contains " + std::to_string(N) + " initial unknowns as per the variable definitions.");
-    badInitialUnknowns = true;
+    fmi2_xml_variable_t* var_ModelStructure = fmi2_import_get_variable(initialUnknowns_ModelStructure, i);
+    size_t originalIndex = fmi2_import_get_variable_original_order(var_ModelStructure);
+    setA_ModelStructure.insert(originalIndex);
   }
 
-  for (size_t i=0; i < N_fmilib; ++i)
+  std::string missing_unknowns = "";
+  for (auto &v : allVariables)
   {
-    fmi2_xml_variable_t* var_fmilib = fmi2_import_get_variable(initialUnknowns, i);
-    size_t originalIndex = fmi2_import_get_variable_original_order(var_fmilib);
+    if (v.isInitialUnknown() && setA_ModelStructure.find(v.getIndex()) == setA_ModelStructure.end())
+    {
+      badInitialUnknowns = true;
+      missing_unknowns += "\n  * " + std::to_string(v.getIndex()+1) + ": " + std::string(v.getCref()) + " is missing";
+    }
+  }
+
+  for (size_t i=0; i < N_ModelStructure; ++i)
+  {
+    fmi2_xml_variable_t* var_ModelStructure = fmi2_import_get_variable(initialUnknowns_ModelStructure, i);
+    size_t originalIndex = fmi2_import_get_variable_original_order(var_ModelStructure);
     const Variable& var_oms = allVariables[originalIndex];
-    oms::ComRef name_fmilib(fmi2_import_get_variable_name(var_fmilib));
+    oms::ComRef name_fmilib(fmi2_import_get_variable_name(var_ModelStructure));
 
     if (var_oms.getCref() != name_fmilib)
     {
-      if (!Flags::IgnoreInitialUnknowns())
-        logWarning("[" + std::string(getCref()) + ": " + getPath() + "] Variable " + std::string(var_oms.getCref()) + " with index " + std::to_string(originalIndex+1) + " could not be found.");
+      missing_unknowns += "\n  * " + std::to_string(originalIndex+1) + ": " + std::string(var_oms.getCref()) + " could not be found";
       badInitialUnknowns = true;
     }
     else if (!var_oms.isInitialUnknown())
     {
-      if (!Flags::IgnoreInitialUnknowns())
-        logWarning("[" + std::string(getCref()) + ": " + getPath() + "] Variable " + std::string(var_oms.getCref()) + " with index " + std::to_string(originalIndex+1) + " is falsely listed as initial unknown.");
+      missing_unknowns += "\n  * " + std::to_string(originalIndex+1) + ": " + std::string(var_oms.getCref()) + " is wrongly listed";
       badInitialUnknowns = true;
     }
   }
 
-  fmi2_import_free_variable_list(initialUnknowns);
+  if (badInitialUnknowns && !Flags::IgnoreInitialUnknowns() && N_ModelStructure > 0)
+    logWarning("[" + std::string(getCref()) + ": " + getPath() + "] The FMU lists " + std::to_string(N_ModelStructure) + " initial unknowns and exposes " + std::to_string(N) + " initial unknowns." + missing_unknowns);
+
+  fmi2_import_free_variable_list(initialUnknowns_ModelStructure);
 
   if (badInitialUnknowns)
   {
-    if(!Flags::IgnoreInitialUnknowns())
+    if(!Flags::IgnoreInitialUnknowns() && N_ModelStructure > 0)
       logInfo("[" + std::string(getCref()) + ": " + getPath() + "] The FMU contains bad initial unknowns. This might cause problems, e.g. wrong simulation results.");
     else
     {
-      logWarning("[" + std::string(getCref()) + ": " + getPath() + "] The dependencies of the initial unknowns defined in the FMU are ignored because the flag --ignoreInitialUnknowns is active. Instead, all the initial unknowns will depend on all inputs.");
+      if (N_ModelStructure > 0)
+        logWarning("[" + std::string(getCref()) + ": " + getPath() + "] The dependencies of the initial unknowns defined in the FMU are ignored because the flag --ignoreInitialUnknowns is active. Instead, all the initial unknowns will depend on all inputs.");
       for (int i=0; i < N; i++)
       {
         logDebug(std::string(getCref()) + ": " + getPath() + " initial unknown " + std::string(initialUnknownsGraph.getNodes()[i]) + " depends on all inputs");
-        for (int j=0; j < inputs.size(); j++)
-          initialUnknownsGraph.addEdge(inputs[j].makeConnector(this->getFullCref()), initialUnknownsGraph.getNodes()[i]);
+        for (const auto& j : inputs)
+          initialUnknownsGraph.addEdge(allVariables[j].makeConnector(this->getFullCref()), initialUnknownsGraph.getNodes()[i]);
       }
       return oms_status_ok;
     }
@@ -440,7 +476,7 @@ oms_status_enu_t oms::ComponentFMUCS::initializeDependencyGraph_initialUnknowns(
   //  buffer += "factorKind[" + std::to_string(i) + "] = " + std::string(fmi2_dependency_factor_kind_to_string((fmi2_dependency_factor_kind_enu_t)factorKind[i])) + "\n";
   //logInfo(buffer);
 
-  for (int i=0; i < N_fmilib; i++)
+  for (int i=0; i < N_ModelStructure; i++)
   {
     if (startIndex[i] == startIndex[i+1])
     {
@@ -449,8 +485,8 @@ oms_status_enu_t oms::ComponentFMUCS::initializeDependencyGraph_initialUnknowns(
     else if ((startIndex[i] + 1 == startIndex[i+1]) && (dependency[startIndex[i]] == 0))
     {
       logDebug(std::string(getCref()) + ": " + getPath() + " initial unknown " + std::string(initialUnknownsGraph.getNodes()[i]) + " depends on all inputs");
-      for (int j = 0; j < inputs.size(); j++)
-        initialUnknownsGraph.addEdge(inputs[j].makeConnector(this->getFullCref()), initialUnknownsGraph.getNodes()[i]);
+      for (const auto& j : inputs)
+        initialUnknownsGraph.addEdge(allVariables[j].makeConnector(this->getFullCref()), initialUnknownsGraph.getNodes()[i]);
     }
     else
     {
@@ -472,7 +508,7 @@ oms_status_enu_t oms::ComponentFMUCS::initializeDependencyGraph_initialUnknowns(
 
 oms_status_enu_t oms::ComponentFMUCS::initializeDependencyGraph_outputs()
 {
-  if (outputsGraph.getEdges().size() > 0)
+  if (outputsGraph.getEdges().connections.size() > 0)
   {
     logError(std::string(getCref()) + ": " + getPath() + " is already initialized.");
     return oms_status_error;
@@ -489,17 +525,18 @@ oms_status_enu_t oms::ComponentFMUCS::initializeDependencyGraph_outputs()
     return oms_status_ok;
   }
 
-  for (int i = 0; i < outputs.size(); i++)
+  for (int i=0; i < outputs.size(); i++)
   {
+    Variable& output = allVariables[outputs[i]];
     if (startIndex[i] == startIndex[i + 1])
     {
-      logDebug(std::string(getCref()) + ": " + getPath() + " output " + std::string(outputs[i]) + " has no dependencies");
+      logDebug(std::string(getCref()) + ": " + getPath() + " output " + std::string(output) + " has no dependencies");
     }
     else if ((startIndex[i] + 1 == startIndex[i + 1]) && (dependency[startIndex[i]] == 0))
     {
-      logDebug(std::string(getCref()) + ": " + getPath() + " output " + std::string(outputs[i]) + " depends on all");
-      for (int j = 0; j < inputs.size(); j++)
-        outputsGraph.addEdge(inputs[j].makeConnector(this->getFullCref()), outputs[i].makeConnector(this->getFullCref()));
+      logDebug(std::string(getCref()) + ": " + getPath() + " output " + std::string(output) + " depends on all");
+      for (const auto& j : inputs)
+        outputsGraph.addEdge(allVariables[j].makeConnector(this->getFullCref()), output.makeConnector(this->getFullCref()));
     }
     else
     {
@@ -507,11 +544,11 @@ oms_status_enu_t oms::ComponentFMUCS::initializeDependencyGraph_outputs()
       {
         if (dependency[j] < 1 || dependency[j] > allVariables.size())
         {
-          logWarning("Output " + std::string(outputs[i]) + " has bad dependency on variable with index " + std::to_string(dependency[j]) + " which couldn't be resolved");
+          logWarning("Output " + std::string(output) + " has bad dependency on variable with index " + std::to_string(dependency[j]) + " which couldn't be resolved");
           return logError(std::string(getCref()) + ": erroneous dependencies detected in modelDescription.xml");
         }
-        logDebug(std::string(getCref()) + ": " + getPath() + " output " + std::string(outputs[i]) + " depends on " + std::string(allVariables[dependency[j] - 1]));
-        outputsGraph.addEdge(allVariables[dependency[j] - 1].makeConnector(this->getFullCref()), outputs[i].makeConnector(this->getFullCref()));
+        logDebug(std::string(getCref()) + ": " + getPath() + " output " + std::string(output) + " depends on " + std::string(allVariables[dependency[j] - 1]));
+        outputsGraph.addEdge(allVariables[dependency[j] - 1].makeConnector(this->getFullCref()), output.makeConnector(this->getFullCref()));
       }
     }
   }
@@ -540,7 +577,8 @@ oms_status_enu_t oms::ComponentFMUCS::instantiate()
     {
       for (const auto &res : it.allresources)
       {
-        setResourcesHelper1(res.second);
+        if (res.second.linkResources) // set values only if resources are linked in ssd
+          setResourcesHelper1(res.second);
       }
     }
   }
@@ -605,32 +643,41 @@ oms_status_enu_t oms::ComponentFMUCS::setResourcesHelper2(Values values)
     {
       for (const auto &v : res.second.booleanStartValues)
       {
-        oms::ComRef tail(v.first);
-        oms::ComRef head = tail.pop_front();
-        if (head == getCref())
+        if (res.second.linkResources) // set values only if resources are linked in ssd
         {
-          if (oms_status_ok != setBoolean(tail, v.second))
-            return logError("Failed to set start value for " + std::string(v.first));
+          oms::ComRef tail(v.first);
+          oms::ComRef head = tail.pop_front();
+          if (head == getCref())
+          {
+            if (oms_status_ok != setBoolean(tail, v.second))
+              return logError("Failed to set start value for " + std::string(v.first));
+          }
         }
       }
       for (const auto &v : res.second.integerStartValues)
       {
-        oms::ComRef tail(v.first);
-        oms::ComRef head = tail.pop_front();
-        if (head == getCref())
+        if (res.second.linkResources) // set values only if resources are linked in ssd
         {
-          if (oms_status_ok != setInteger(tail, v.second))
-            return logError("Failed to set start value for " + std::string(v.first));
+          oms::ComRef tail(v.first);
+          oms::ComRef head = tail.pop_front();
+          if (head == getCref())
+          {
+            if (oms_status_ok != setInteger(tail, v.second))
+              return logError("Failed to set start value for " + std::string(v.first));
+          }
         }
       }
       for (const auto &v : res.second.realStartValues)
       {
-        oms::ComRef tail(v.first);
-        oms::ComRef head = tail.pop_front();
-        if (head == getCref())
+        if (res.second.linkResources) // set values only if resources are linked in ssd
         {
-          if (oms_status_ok != setReal(tail, v.second))
-            return logError("Failed to set start value for " + std::string(v.first));
+          oms::ComRef tail(v.first);
+          oms::ComRef head = tail.pop_front();
+          if (head == getCref())
+          {
+            if (oms_status_ok != setReal(tail, v.second))
+              return logError("Failed to set start value for " + std::string(v.first));
+          }
         }
       }
     }
@@ -655,23 +702,62 @@ oms::ComRef oms::ComponentFMUCS::getValidCref(ComRef cref)
   return tail;
 }
 
-oms_status_enu_t oms::ComponentFMUCS::addResources(std::string& filename)
+oms_status_enu_t oms::ComponentFMUCS::newResources(const std::string& ssvFilename, const std::string& ssmFilename, bool externalResources)
 {
   Values resources;
+  if (externalResources) // check of external resources and override the start values with new references
+  {
+    Snapshot snapshot;
+    snapshot.importResourceFile(ssvFilename, getModel().getTempDirectory() + "/resources");
+
+    // import ssm file, if provided
+    if (!ssmFilename.empty())
+      snapshot.importResourceFile(ssmFilename, getModel().getTempDirectory() + "/resources");
+
+    if (oms_status_ok != resources.importFromSnapshot(snapshot, ssvFilename, ssmFilename))
+      return logError("referenceResources failed for \"" + std::string(getFullCref()) + ":" + ssvFilename + "\"");
+  }
+
   if (!values.hasResources())
   {
-    resources.allresources[filename] = resources;
+    if(!ssmFilename.empty())
+      resources.ssmFile = "resources/" + ssmFilename;
+    // copy modeldescriptionVariableUnits to ssv resources which will be used to export units
+    resources.modelDescriptionVariableUnits = values.modelDescriptionVariableUnits;
+    // copy modeldescriptionVariableUnitDefinitions to ssv resources which will be used to export unit definitions
+    resources.modeldescriptionUnitDefinitions = values.modeldescriptionUnitDefinitions;
+    resources.allresources["resources/" + ssvFilename] = resources;
     values.parameterResources.push_back(resources);
   }
   else
   {
     // generate empty ssv file, if more resources are added to same level
-    values.parameterResources[0].allresources[filename] = resources;
+    if(!ssmFilename.empty())
+      resources.ssmFile = "resources/" + ssmFilename;
+    // copy modeldescriptionVariableUnits to ssv resources which will be used to export units
+    resources.modelDescriptionVariableUnits = values.modelDescriptionVariableUnits;
+    values.parameterResources[0].allresources["resources/" + ssvFilename] = resources;
   }
 
   return oms_status_ok;
 }
 
+oms_status_enu_t oms::ComponentFMUCS::deleteReferencesInSSD(const std::string& filename)
+{
+  if (values.hasResources())
+    return values.deleteReferencesInSSD(filename);
+
+  return oms_status_error;
+}
+
+oms_status_enu_t oms::ComponentFMUCS::deleteResourcesInSSP(const std::string& filename)
+{
+  if (values.hasResources())
+    return values.deleteResourcesInSSP(filename);
+
+  return oms_status_error;
+}
+
 oms_status_enu_t oms::ComponentFMUCS::initialize()
 {
   clock.reset();
@@ -1072,6 +1158,215 @@ oms_status_enu_t oms::ComponentFMUCS::getReal(const ComRef& cref, double& value)
   return getReal(vr, value);
 }
 
+oms_status_enu_t oms::ComponentFMUCS::getString(const fmi2_value_reference_t& vr, std::string& value)
+{
+  CallClock callClock(clock);
+  fmi2_string_t str;
+  if (fmi2_status_ok != fmi2_import_get_string(fmu, &vr, 1, &str))
+    return oms_status_error;
+
+  value = std::string(str);
+
+  return oms_status_ok;
+}
+
+oms_status_enu_t oms::ComponentFMUCS::getString(const ComRef& cref, std::string& value)
+{
+  CallClock callClock(clock);
+
+  if (oms_modelState_virgin == getModel().getModelState())
+  {
+    // check for start values exist, priority over modeldescription.xml start values
+    if (values.hasResources())  // search in local resources
+    {
+      if (oms_status_ok == values.getStringResources(cref, value, false, oms_modelState_virgin))
+      {
+        return oms_status_ok;
+      }
+      // search in modelDescription.xml
+      else if (oms_status_ok == values.getStringFromModeldescription(cref, value))
+      {
+        return oms_status_ok;
+      }
+
+      return logError("no start value set or available for signal: " + std::string(getFullCref() + cref));
+    }
+    else if (getParentSystem() && getParentSystem()->getValues().hasResources())  // search in root resources
+    {
+      if (oms_status_ok == getParentSystem()->getValues().getStringResources(getCref()+cref, value, false, oms_modelState_virgin))
+      {
+        return oms_status_ok;
+      }
+      // search in modelDescription.xml
+      else if(oms_status_ok == values.getStringFromModeldescription(cref, value))
+      {
+        return oms_status_ok;
+      }
+
+      return logError("no start value set or available for signal: " + std::string(getFullCref() + cref));
+    }
+    else if (getParentSystem()->getParentSystem() && getParentSystem()->getParentSystem()->getValues().hasResources())  // search in top level root resources
+    {
+      if (oms_status_ok == getParentSystem()->getParentSystem()->getValues().getStringResources(getCref()+cref, value, false, oms_modelState_virgin))
+      {
+        return oms_status_ok;
+      }
+      // search in modelDescription.xml
+      else if(oms_status_ok == values.getStringFromModeldescription(cref, value))
+      {
+        return oms_status_ok;
+      }
+
+      return logError("no start value set or available for signal: " + std::string(getFullCref() + cref));
+    }
+    else // search inline
+    {
+      // check for start values exist, priority over modeldescription.xml start values
+      if (oms_status_ok == values.getString(cref, value))
+      {
+        return oms_status_ok;
+      }
+      else
+      {
+        return values.getStringFromModeldescription(cref, value);
+      }
+    }
+  }
+
+  int j=-1;
+  for (size_t i = 0; i < allVariables.size(); i++)
+  {
+    if (allVariables[i].getCref() == cref && allVariables[i].isTypeString())
+    {
+      j = i;
+      break;
+    }
+  }
+
+  if (!fmu || j < 0)
+    return logError_UnknownSignal(getFullCref() + cref);
+
+  fmi2_value_reference_t vr = allVariables[j].getValueReference();
+  return getString(vr, value);
+}
+
+oms_status_enu_t oms::ComponentFMUCS::getDirectionalDerivative(const ComRef& unknownCref, const ComRef& knownCref, double& value)
+{
+  if (!getModel().validState(oms_modelState_instantiated|oms_modelState_initialization|oms_modelState_simulation))
+    return logError_ModelInWrongState(getModel().getCref());
+
+  if (!getFMUInfo()->getProvidesDirectionalDerivative())
+    return logError("FMU \"" + std::string(getFullCref()) + "\" doesn't support directional derivatives (providesDirectionalDerivative = false in modelDescription.xml)");
+
+  int j = -1;
+  for (size_t i = 0; i < allVariables.size(); i++)
+  {
+    if (allVariables[i].getCref() == unknownCref && allVariables[i].isTypeReal())
+    {
+      j = i;
+      break;
+    }
+  }
+
+  // check for knownIndex, if provided
+  int knownIndex = -1;
+  if (!knownCref.isEmpty())
+  {
+    for (size_t i = 0; i < allVariables.size(); i++)
+    {
+      if (allVariables[i].getCref() == knownCref && allVariables[i].isTypeReal())
+      {
+        knownIndex = i;
+        break;
+      }
+    }
+  }
+
+  if (!fmu || j < 0)
+    return logError_UnknownSignal(getFullCref() + unknownCref);
+
+  if (oms_modelState_instantiated == getModel().getModelState() || oms_modelState_initialization == getModel().getModelState())
+  {
+    // check index exist in ModelStructure inititalUnknowns
+    auto index = values.modelStructureInitialUnknowns.find(j+1);
+    if (index == values.modelStructureInitialUnknowns.end())
+      return logError("Signal \"" + std::string(getFullCref() + unknownCref) + "\" could not be resolved to an <InitialUnknowns> index in <ModelStructure>");
+
+    //get dependencylist from <InitialUnknowns> in <ModelStructure>
+    std::vector<int> dependencyList = values.modelStructureInitialUnknowns[j+1];
+    getDirectionalDerivativeHeper(j, knownIndex, dependencyList, value);
+  }
+
+  if (oms_modelState_simulation == getModel().getModelState())
+  {
+    if (!allVariables[j].isOutput() && !allVariables[j].isState() && !allVariables[j].isDer())
+      return logError("Signal \"" + std::string(getFullCref() + unknownCref) + "\" could not be resolved to an output or state or derivates after initalization");
+
+    // <Outputs>
+    if (allVariables[j].isOutput())
+    {
+      // check index exist in ModelStructure inititalUnknowns
+      auto index = values.modelStructureOutputs.find(j + 1);
+      if (index == values.modelStructureOutputs.end())
+        return logError("Signal \"" + std::string(getFullCref() + unknownCref) + "\" could not be resolved to an <Outputs> index in <ModelStructure>");
+
+      // get dependencylist from <Outputs> in <ModelStructure>
+      std::vector<int> dependencyList = values.modelStructureOutputs[j + 1];
+      getDirectionalDerivativeHeper(j, knownIndex, dependencyList, value);
+    }
+    // <Derivatives>
+    if (allVariables[j].isState() || allVariables[j].isDer())
+    {
+      // check index exist in ModelStructure inititalUnknowns
+      auto index = values.modelStructureDerivatives.find(j + 1);
+      if (index == values.modelStructureDerivatives.end())
+        return logError("Signal \"" + std::string(getFullCref() + unknownCref) + "\" could not be resolved to an <Derivatives> index in <ModelStructure>");
+
+      // get dependencylist from <Derivatives> in <ModelStructure>
+      std::vector<int> dependencyList = values.modelStructureDerivatives[j + 1];
+      getDirectionalDerivativeHeper(j, knownIndex, dependencyList, value);
+    }
+  }
+
+  // fmi2_value_reference_t vr_unknown[1] = {5};
+  // fmi2_value_reference_t vr_known[4] = {0, 2, 3, 4};
+  // fmi2_real_t dvknown[4] = {1.0, 1.0, 1.0, 1.0};
+  // fmi2_real_t val;
+  // std::cout << "Get directional derivative_static_1: " << val << std::endl;
+  // fmi2_import_get_directional_derivative(fmu, vr_unknown, 1, vr_known, 4, dvknown, &val);
+  // std::cout << "\nGet directional derivative_static_2: " << val << std::endl;
+
+  return oms_status_ok;
+}
+
+oms_status_enu_t oms::ComponentFMUCS::getDirectionalDerivativeHeper(const int unknownIndex, const int knownIndex, const std::vector<int> &dependencyList, double &value)
+{
+  fmi2_value_reference_t vr_unknown = allVariables[unknownIndex].getValueReference();
+  fmi2_value_reference_t *vr_known = (fmi2_value_reference_t *)calloc(dependencyList.size(), sizeof(fmi2_value_reference_t *));
+  fmi2_real_t *dvknown = (fmi2_real_t *)calloc(dependencyList.size(), sizeof(fmi2_real_t *));
+
+  for (int i = 0; i < dependencyList.size(); i++)
+  {
+    vr_known[i] = allVariables[dependencyList[i] - 1].getValueReference();
+
+    // The knownIndex is < 0 if not specified. In this case, we
+    // calculate the sum of the row, which means we set all seed
+    // values to 1.0. Otherwise we just set the explicitly provided
+    // element to 1.0.
+    if (knownIndex < 0 || (dependencyList[i] == knownIndex + 1))
+      dvknown[i] = 1.0;
+    else
+      dvknown[i] = 0.0;
+  }
+
+  fmi2_import_get_directional_derivative(fmu, &vr_unknown, 1, vr_known, dependencyList.size(), dvknown, &value);
+
+  free(vr_known);
+  free(dvknown);
+
+  return oms_status_ok;
+}
+
 oms_status_enu_t oms::ComponentFMUCS::getRealOutputDerivative(const ComRef& cref, SignalDerivative& der)
 {
   CallClock callClock(clock);
@@ -1274,6 +1569,85 @@ oms_status_enu_t oms::ComponentFMUCS::setReal(const ComRef& cref, double value)
   return oms_status_ok;
 }
 
+oms_status_enu_t oms::ComponentFMUCS::setString(const ComRef& cref, const std::string& value)
+{
+  CallClock callClock(clock);
+  int j=-1;
+  for (size_t i = 0; i < allVariables.size(); i++)
+  {
+    if (allVariables[i].getCref() == cref && allVariables[i].isTypeString())
+    {
+      j = i;
+      break;
+    }
+  }
+
+  if (!fmu || j < 0)
+    return logError_UnknownSignal(getFullCref() + cref);
+
+  if (getModel().validState(oms_modelState_virgin|oms_modelState_enterInstantiation|oms_modelState_instantiated))
+    if (allVariables[j].isCalculated() || allVariables[j].isIndependent())
+      return logWarning("It is not allowed to provide a start value if initial=\"calculated\" or causality=\"independent\".");
+
+  if (oms_modelState_virgin == getModel().getModelState())
+  {
+    // check for local resources available
+    if (values.hasResources())
+    {
+      return values.setStringResources(cref, value, getFullCref(), false, oms_modelState_virgin);
+    }
+    // check for resources in root
+    else if (getParentSystem() && getParentSystem()->getValues().hasResources())
+    {
+      return getParentSystem()->getValues().setStringResources(getCref()+cref, value, getParentSystem()->getFullCref(), false, oms_modelState_virgin);
+    }
+    // check for resources in top level root
+    else if (getParentSystem()->getParentSystem() && getParentSystem()->getParentSystem()->getValues().hasResources())
+    {
+      return getParentSystem()->getParentSystem()->getValues().setStringResources(getCref()+cref, value, getParentSystem()->getParentSystem()->getFullCref(), false, oms_modelState_virgin);
+    }
+    else
+    {
+      //inline parameter settings
+      values.setString(cref, value);
+    }
+  }
+  else
+  {
+    fmi2_value_reference_t vr = allVariables[j].getValueReference();
+    fmi2_string_t value_ = value.c_str();
+    if (fmi2_status_ok != fmi2_import_set_string(fmu, &vr, 1, &value_))
+      return oms_status_error;
+  }
+
+  return oms_status_ok;
+}
+
+oms_status_enu_t oms::ComponentFMUCS::setUnit(const ComRef &cref, const std::string &value)
+{
+  // check for local resources available
+  if (values.hasResources())
+  {
+    return values.setUnitResources(cref, value, getFullCref());
+  }
+  // check for resources in root
+  else if (getParentSystem() && getParentSystem()->getValues().hasResources())
+  {
+    return getParentSystem()->getValues().setUnitResources(getCref() + cref, value, getParentSystem()->getFullCref());
+  }
+  // check for resources in top level root
+  else if (getParentSystem()->getParentSystem() && getParentSystem()->getParentSystem()->getValues().hasResources())
+  {
+    return getParentSystem()->getParentSystem()->getValues().setUnitResources(getCref() + cref, value, getParentSystem()->getParentSystem()->getFullCref());
+  }
+  else
+  {
+    // inline unit settings
+    values.setUnit(cref, value);
+  }
+  return oms_status_ok;
+}
+
 oms_status_enu_t oms::ComponentFMUCS::deleteStartValue(const ComRef& cref)
 {
   // check for local resources
