@@ -38,6 +38,8 @@
 #include "OMSimulator.h"
 #include "Scope.h"
 #include "System.h"
+#include "Util.h"
+
 #include <iomanip>
 #include <iostream>
 #include <RegEx.h>
@@ -62,31 +64,36 @@ oms::Flags::~Flags()
 void oms::Flags::setDefaults()
 {
   addParametersToCSV = false;
-  algLoopSolver = oms_alg_solver_fixedpoint;
+  algLoopSolver = oms_alg_solver_kinsol;
   defaultModeIsCS = false;
   deleteTempFiles = true;
+  directionalDerivatives = true;
+  dumpAlgLoops = false;
   emitEvents = true;
-  exportParametersInline = true;
   ignoreInitialUnknowns = false;
+  initialStepSize = 1e-6;
   inputExtrapolation = false;
   intervals = 100;
   masterAlgorithm = oms_solver_wc_ma;
   maxEventIteration = 100;
+  maximumStepSize = 1e-3;
   maxLoopIteration = 10;
+  minimumStepSize = 1e-12;
   numProcs = 1;
   progressBar = false;
   realTime = false;
   resultFile = "<default>";
-  skipCSVHeader = false;
+  skipCSVHeader = true;
   solver = oms_solver_sc_cvode;
   solverStats = false;
   startTime = 0.0;
   stopTime = 1.0;
   stripRoot = false;
-  suppressPath = false;
+  suppressPath = true;
   timeout = 0.0;
   tolerance = 1e-4;
   wallTime = false;
+  zeroNominal = false;
 }
 
 oms::Flags& oms::Flags::GetInstance()
@@ -202,15 +209,21 @@ oms_status_enu_t oms::Flags::DeleteTempFiles(const std::string& value)
   return oms_status_ok;
 }
 
-oms_status_enu_t oms::Flags::EmitEvents(const std::string& value)
+oms_status_enu_t oms::Flags::DirectionalDerivatives(const std::string& value)
 {
-  GetInstance().emitEvents = (value == "true");
+  GetInstance().directionalDerivatives = (value == "true");
   return oms_status_ok;
 }
 
-oms_status_enu_t oms::Flags::ExportParametersInline(const std::string& value)
+oms_status_enu_t oms::Flags::DumpAlgLoops(const std::string& value)
 {
-  GetInstance().exportParametersInline = (value == "true");
+  GetInstance().dumpAlgLoops = (value == "true");
+  return oms_status_ok;
+}
+
+oms_status_enu_t oms::Flags::EmitEvents(const std::string& value)
+{
+  GetInstance().emitEvents = (value == "true");
   return oms_status_ok;
 }
 
@@ -361,12 +374,6 @@ oms_status_enu_t oms::Flags::ResultFile(const std::string& value)
   return oms_status_ok;
 }
 
-oms_status_enu_t oms::Flags::SetInputDerivatives(const std::string& value)
-{
-  logWarning("--setInputDerivatives is deprecated; use --inputExtrapolation instead");
-  return InputExtrapolation(value);
-}
-
 oms_status_enu_t oms::Flags::SkipCSVHeader(const std::string& value)
 {
   GetInstance().skipCSVHeader = (value == "true");
@@ -397,6 +404,37 @@ oms_status_enu_t oms::Flags::StartTime(const std::string& value)
   return oms_status_ok;
 }
 
+std::vector<std::string> split(const std::string& s, char delim)
+{
+  std::vector<std::string> result;
+  std::stringstream ss(s);
+  std::string item;
+
+  while (getline(ss, item, delim))
+    result.push_back(item);
+
+  return result;
+}
+
+oms_status_enu_t oms::Flags::StepSize(const std::string& value)
+{
+  std::vector<std::string> options = split(value, ',');
+
+  for (const auto& option : options)
+    if (atof(option.c_str()) <= 0.0)
+      return logError("The step size value must be a greater than zero: " + option);
+
+  if (options.size() > 1)
+  {
+    GetInstance().initialStepSize = atof(options[0].c_str());
+    GetInstance().minimumStepSize = atof(options[1].c_str());
+    GetInstance().maximumStepSize = atof(options[2].c_str());
+  }
+  else
+    GetInstance().maximumStepSize = atof(options[0].c_str());
+  return oms_status_ok;
+}
+
 oms_status_enu_t oms::Flags::StopTime(const std::string& value)
 {
   GetInstance().stopTime = atof(value.c_str());
@@ -450,3 +488,9 @@ oms_status_enu_t oms::Flags::WallTime(const std::string& value)
   GetInstance().wallTime = (value == "true");
   return oms_status_ok;
 }
+
+oms_status_enu_t oms::Flags::ZeroNominal(const std::string& value)
+{
+  GetInstance().zeroNominal = (value == "true");
+  return oms_status_ok;
+}
